/*
************************************************************************
**

  The implementation or calls to other implementations of the experiment
  system. See expt.h for descriptions of functions.

  Author: V. B. Balayoghan (vbb@cs.utexas.edu)
  Modified by: Scott F. Kaplan (sfkaplan@cs.utexas.edu)
  Date:   June 1997
  Note: This experiment module is specific to compressed paging.

**
************************************************************************
*/


#include <string.h>
#include "expt.h"
#include "wrap_write.h"



/*
** The initialization call made from InitVMTrace() in the tracer
** if it was compiled with tracing enabled and the environment
** variable VMT_EXPT is defined at runtime. The "experiment system"
** (page compression etc) is expected to initialize itself when this
** call is made.
*/
void
ExptInitSystem () {}



/*
** The call made from the tracing system when a protected page is
** faulted on and has been uncompressed.
*/
void
ExptFetchEvent (PageInfoElement* pageInfo) {

  /* When a page is brought into the unprotected set, we want to hold
     onto a copy of its state so that it can be dumped when that page
     is evicted from the unprotected set. */
  if (pageInfo->pageAddress != (caddr_t)0) {
    memcpy((void*)pageInfo->fetchedPageImage,
	   (void*)pageInfo->pageAddress,
	   PAGESIZE);

  }
}



/*
** The call made from the tracing system when a protected page is
** evicted from the FIFO queue and is about to be compressed.
*/
void
ExptEvictEvent (PageInfoElement* pageInfo) {}



/* The call made when the tracing system wants to output the stored
   experimental data about some page. */
void
ExptOutputData (PageInfoElement* pageInfo, int fd) {

  /* Output the image of this page as it was when it was brought into
     the unprotected set. */
  write_VMT(fd, (char*)pageInfo->fetchedPageImage, PAGESIZE);

  /* If the page was dirtied, output its current state. */
  if (pageInfo->dirty) {

    write_VMT(fd, (char*)pageInfo->pageAddress, PAGESIZE);

  }
}



/*
** The termination call made from ShutdownVMTrace() in the tracer
** which is in turn called from the wrapper for exit().
** The experiment system can use this call to print out summaries
** etc.
*/
void ExptShutdownSystem (int fd) {

  char terminatingChar = 'Q';
  /* Output a terminating character for the trace.  [Note] This tag
     shouldn't be necessary!  I can't get feof() to work properly
     when reading from a pipe, so any program that takes this trace as
     an input through a pipe needs an explicit terminating tag. */
  write_VMT(fd, &terminatingChar, 1);

}
