/*
************************************************************************
**

  The header file for the experiments to be run under tracing.

  Author: V. B. Balayoghan (vbb@cs.utexas.edu)
  Modified by: Scott F. Kaplan (sfkaplan@cs.utexas.edu)
  Date:   February 1997

**
************************************************************************
*/


#ifndef _expt_h
#define _expt_h

#include "traceContext.h"


/*
** The initialization call made from InitVMTrace() in the tracer
** if it was compiled with tracing enabled and the environment
** variable VMT_EXPT is defined at runtime. The "experiment system"
** (page compression etc) is expected to initialize itself when this
** call is made.
*/
void ExptInitSystem ();


/*
** The call made from the tracing system when a protected page is
** faulted on and has been uncompressed.
*/
void ExptFetchEvent (PageInfoElement* pageInfo);


/*
** The call made from the tracing system when a protected page is
** evicted from the FIFO queue and is about to be compressed.
*/
void ExptEvictEvent (PageInfoElement* pageInfo);



/* The call made when the tracing system wants to output the stored
   experimental data about some page. */
void ExptOutputData (PageInfoElement* pageInfo, int fd);



/*
** The termination call made from ShutdownVMTrace() in the tracer
** which is in turn called from the wrapper for exit().
** The experiment system can use this call to print out summaries
** etc.
*/
void ExptShutdownSystem (int fd);



#endif /* _expt_h */
