
/*
************************************************************************
**

  The wrapper for open()

  Author: Scott F. Kaplan (sfkaplan@cs.utexas.edu)
  Date:   July 1996
  Last modified: June 1998

**
************************************************************************
*/


#include <stdarg.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include "wrap_open.h"

#define BUFFER_SIZE 256

/* Note: The Solaris 5.x libc.a management of open is the most bizzare
   thing that I've ever seen.  Two object files must be examined:
   open.o and libc_open.o.  It seems that all calls to open eventually
   rely on the "_open" symbol in open.o.  However, there is also a
   weak "_open" in libc_open.o that resolves to "_libc_open".
   Irrespective of the "_open" chosen for resolution by the linker, by
   wrapping it, we guarantee that a wrapper will be used.  We also
   wrap "__open", as it is a weak symbol that resolves to the code of
   "_open" in open.o.  Finally, we do NOT wrap "open" in libc_open.o,
   as it is resolved to "_libc_open", which in turn calls "__open" in
   open.o, which is wrapped.

   I do hope that Sun hires smarter people in the future. */

#ifdef __svr4__
#pragma weak __wrap__open = __wrap___open
#else /* not __svr4__ */
#ifdef __linux__
#pragma weak __wrap___libc_open = __wrap___open
#pragma weak __wrap_open = __wrap___open
#endif /* __linux__ */
#endif /* __svr4__ */

int __wrap___open (const char *pathname, int flags, ...) {

  int returnValue;
  va_list ap;
  mode_t mode;
  char safePathname[BUFFER_SIZE];

  /* Copy the filename into a safe space. */
  strcpy(safePathname,pathname);

  /* Is one of the flags O_CREAT (create the file if it doesn't exist)? */
  if (flags && O_CREAT) {
   
    /* The O_CREAT flag was specified, so there must be an additional
       "mode" argument.  Extract it, and call the real "open" using
       it. */
    va_start(ap,flags);
    mode = va_arg(ap,mode_t);
    va_end(ap);
    returnValue = __real___open(pathname, flags, mode);
 
  } else {

    /* The O_CREAT flag was not specified.  Call the real "open"
       without any mode specification. */
    returnValue = __real___open(safePathname, flags);

  }

  /* Return whatever the result of the real open call was. */
  return returnValue;
}
