/*
************************************************************************
**

  The wrapper for the sbrk() system call.

  Author: V. B. Balayoghan (vbb@cs.utexas.edu)
  Date: October 1994
  Modified by: Scott F. Kaplan <sfkaplan@cs.utexas.edu>
  Last modification: September 1998
  Original version based on a program by Atif Choudary that used a
  customized malloc() implementation to track heap allocations.

**
************************************************************************ */


#include <sys/types.h>
#include <sys/mman.h>
#include <stdio.h>
#include <errno.h>
#include "traceContext.h"
#include "wrap_sbrk.h"


/*
** The wrapper for sbrk()
*/
#ifdef __svr4__
#pragma weak __wrap__sbrk = __wrap_sbrk
#else /* not __svr4__ */
#ifdef __linux__
#pragma weak __wrap___sbrk = __wrap_sbrk
#endif /* __linux__ */
#endif /* __svr4__ */

void* __wrap_sbrk (int increment) {
  caddr_t rval;
  caddr_t new_abrk;

  /* If tracing is enabled, then manage the allocation and protection
     of heap pages.  Otherwise, act as a dumb wrapper by calling sbrk
     and simply returning the result. */
  if (tcc.status == ENABLED) {

    tcc.tc->ubrk+= increment;

    if (tcc.tc->ubrk > tcc.tc->abrk) {

      new_abrk =
	(caddr_t)(((unsigned long)(tcc.tc->ubrk - 1) &
		   (unsigned long)PAGE_BEG_MASK) +
		  PAGESIZE);

      rval = __real_sbrk(new_abrk - tcc.tc->abrk);
      if (rval == (void*)-1)
	return rval;

      if (mprotect(tcc.tc->abrk,
		   new_abrk-tcc.tc->abrk,
		   PROT_NONE) == -1) {
	perror("VMTrace: Error protecting new heap region");
	abort();
      }

      tcc.tc->abrk = new_abrk;
    }

    return (tcc.tc->ubrk - increment);

  } else {

    /* Tracing is not enabled, so simply return the result of calling
       the real sbrk. */
    return (__real_sbrk(increment));

  }
}
