
/*
************************************************************************
**

  The wrapper for write()

  Author: V. B. Balayoghan (vbb@cs.utexas.edu)
  Modified by: Scott F. Kaplan (sfkaplan@cs.utexas.edu)
  Date:   June 1998

**
************************************************************************
*/


#include <errno.h>
#include "wrap_write.h"

#define BUFSIZE 256

#ifdef __svr4__

extern int __real__write (int fd, char* buf, int len);
#pragma weak __wrap__libc_write = __wrap__write
int __wrap__write (int fd, char* buf, int len)

#else /* not __svr4__ */
#ifdef __linux__

extern int __real_write (int fd, char* buf, int len);
#pragma weak __wrap___libc_write = __wrap_write
#pragma weak __wrap___write = __wrap_write
int __wrap_write (int fd, char* buf, int len)

#endif /* __linux__ */
#endif /* __svr4__ */

{
  char sbuf[BUFSIZE];
  int rval, n_towrite, n_written;
  int i;
  char *s, *t;

  for (i= BUFSIZE; i>=0; i--)
    rval+= sbuf[i];

  n_written= 0;

  s= buf;

  while (len > 0)
    {
      n_towrite= BUFSIZE<len ? BUFSIZE : len;

      for (i=n_towrite, t=sbuf; i>0; i--)
	*t++= *s++;

      rval = write_VMT(fd, sbuf, n_towrite);

      if (rval == n_towrite)
	{
	  n_written+= rval;
	  len-= rval;
	}
      else if (rval >= 0)
	return rval+n_written;
      else if (errno == EAGAIN)
	return n_written>0 ? n_written : -1;
      else
	return rval;
    }

  return n_written;
}



int
write_VMT (int fileDescriptor, char* buffer, int bytesToWrite) {

  int returnValue;

#if defined (__svr4__)
  returnValue = __real__write(fileDescriptor, buffer, bytesToWrite);
#else /* not __svr4__ */
#if defined (__linux__)
  returnValue = __real_write(fileDescriptor, buffer, bytesToWrite);
#endif /* __linux__ */
#endif /* __svr4__ */

  return returnValue;

}
