
/*
************************************************************************
**

  The wrapper for writev()

  Author: V. B. Balayoghan (vbb@cs.utexas.edu)
  Modified by: Scott F. Kaplan (sfkaplan@cs.utexas.edu)
  Date:   July 1996

**
************************************************************************
*/


#include <sys/types.h>
#include <sys/uio.h>
#include <errno.h>
#include "wrap_writev.h"

#define BUFSIZE 1024


#ifdef __svr4__
#pragma weak __wrap__writev = __wrap_writev
int __wrap_writev (int fd, const struct iovec *iov, int iovcnt)
#else /* not __svr4__ */
#ifdef __linux__
#pragma weak __wrap___writev = __wrap_writev
int __wrap_writev (int fd, const struct iovec *iov, size_t iovcnt)
#endif /* __linux__ */
#endif /* __svr4__ */

 {
 /*
 ** The buffer is placed in stack to make the wrapper re-entrant.
 */
 char sbuf[BUFSIZE];
 struct iovec iv;
 int rval, len, n_written, n_towrite;
 int i;
 char *s, *t;

 for (i= BUFSIZE; i>=0; i--)
    rval+= sbuf[i];

 n_written= 0;

 iv.iov_base= (caddr_t)sbuf;

 for (; iovcnt>0; iov++, iovcnt--)
    {
    s= iov->iov_base;
    len= iov->iov_len;

    while (len > 0)
       {
       iv.iov_len= n_towrite= BUFSIZE<len ? BUFSIZE : len;

       for (i=n_towrite, t=sbuf; i>0; i--)
          *t++= *s++;

       rval= __real_writev(fd, &iv, 1);

       if (rval == n_towrite)
	  {
	  n_written+= rval;
	  len-= rval;
	  }
       else
	  {

	  /*
	  ** Check for errors and error codes: EAGAIN ==> retry.
	  */

	  if (rval >= 0)
	     return rval+n_written;
	  else if (rval==-1 && errno==EAGAIN)
	     return n_written>0 ? n_written : -1;
	  else
	     return rval;
          }
       }
    }

 return n_written;
 }
